#include <sys/types.h>
#include <sys/wait.h>
#include <sys/socket.h>
#include <stdio.h>
#include <stdlib.h>
#include <netinet/in.h>
#include <syslog.h>
#include <errno.h>
#include <unistd.h>
#include <signal.h>

#include "tcp_listen.h"
#include "signal.h"
#include "pop3_session.h"
#include "ssl.h"

char * PORT = "110";

void sig_chld(int signo) {
  pid_t pid;
  int stat;
  while ((pid = waitpid(-1,&stat,WNOHANG)) > 0);
}

static void usage(char * argv0) {
  fprintf(stderr,"%s: usage: %s [-s]\n",argv0,argv0);
  exit(EXIT_FAILURE);
}

int main(int argc, char * argv[]) {
  int listenfd, connfd;
  pid_t childpid;
  socklen_t clilen, addrlen;
  struct sockaddr * cliaddr;

  if (argc>1) {
    if (strcmp(argv[1],"-s")==0) {
      set_use_ssl(1);
      PORT = "995"; /* POP3 over SSL */
    } else {
      usage(argv[0]);
    }
  }

  openlog(argv[0], LOG_PID, LOG_MAIL);

  listenfd = Tcp_listen(NULL, PORT, &addrlen);
  cliaddr = malloc(addrlen);
  if (cliaddr==NULL) {
    syslog(LOG_ERR,"virtual memory exhausted. Can't allocate %d bytes: %m",addrlen);
    perror("virtual memory exhausted");
    exit(EXIT_FAILURE);
  }

  Signal(SIGCHLD,sig_chld);

  for (;;) {
    clilen = addrlen;
    if ((connfd = accept(listenfd,cliaddr,&clilen)) < 0) {
      if (errno == EINTR) {
        continue;
      } else {
        syslog(LOG_ERR,"accept error: %m");
        continue;
      }
    }
    if ((childpid = fork()) == 0) {
      close(listenfd);
      pop3_session(connfd);
      exit(0);
    } else if (childpid == -1) {
      syslog(LOG_ERR,"fork error. Can't service a user: %m");
    }
    close(connfd);
  }
  /* NOTREACHED */
  return 0;
}
